@@ -1,12 +1,12 @@
 ;;; parsebib.el --- A library for parsing bib files  -*- lexical-binding: t -*-
 
-;; Copyright (c) 2014-2022 Joost Kremers
+;; Copyright (c) 2014-2024 Joost Kremers
 ;; All rights reserved.
 
 ;; Author: Joost Kremers <joostkremers@fastmail.fm>
 ;; Maintainer: Joost Kremers <joostkremers@fastmail.fm>
 ;; Created: 2014
-;; Version: 4.2
+;; Version: 4.7
 ;; Keywords: text bibtex
 ;; URL: https://github.com/joostkremers/parsebib
 ;; Package-Requires: ((emacs "25.1"))
@@ -44,8 +44,8 @@
 (require 'cl-lib)
 (eval-when-compile (require 'subr-x)) ; for `string-join'.
 (eval-and-compile (unless (fboundp 'json-parse-buffer)
-                    (require 'json)
-                    (defvar json-object-type)))
+                    (require 'json)))
+(defvar json-object-type)
 
 (declare-function json-read "json.el")
 
@@ -333,9 +333,9 @@ See `parsebib-TeX-markup-replacement-alist' and the function
      ;; because doing it properly requires sexp parsing. It will fail
      ;; for cases like \command{\anothercommand{an arg}some text}.
      . ,(rx "\\" (group-n 1 (or (1+ letter) nonl))
-          (: (* blank) (opt (or (: (* (: "[" (* (not "]")) "]"))
-                                 "{" (group-n 2 (0+ (not "}"))) (opt "}"))
-                                (group-n 3 letter))))))
+            (: (* blank) (opt (or (: (* (: "[" (* (not (any "]"))) "]"))
+                                     "{" (group-n 2 (0+ (not (any "}")))) (opt "}"))
+                                  (group-n 3 letter))))))
     (parsebib--replace-literal
      . ,(rx-to-string `(or ,@(mapcar #'car parsebib-TeX-literal-replacement-alist)
                            (1+ blank)))))
@@ -353,9 +353,9 @@ For the common cases of replacing a LaTeX command or a literal
 it is faster to use `parsebib-TeX-command-replacement-alist'
 and `parsebib-TeX-literal-replacement-alist' respectively.")
 
-(defvar parsebib-clean-TeX-markup-excluded-fields '("file"
-                                                    "url"
-                                                    "doi")
+(defvar parsebib-postprocessing-excluded-fields '("file"
+                                                  "url"
+                                                  "doi")
   "List of fields that should not be passed to `parsebib-clean-TeX-markup'.")
 
 (defun parsebib--replace-command-or-accent (string)
@@ -383,7 +383,7 @@ and `parsebib-TeX-literal-replacement-alist' respectively.")
      ;; empty.
      ((and bar (not (equal "" bar))) bar)
      ;; Otherwise clean any optional arguments by discarding them.
-     (t (replace-regexp-in-string (rx "[" (* (not "]")) "]") "" string t t)))))
+     (t (replace-regexp-in-string (rx "[" (* (not (any "]"))) "]") "" string t t)))))
 
 (defun parsebib--replace-literal (string)
   "Look up the replacement text for literal STRING."
@@ -395,8 +395,7 @@ and `parsebib-TeX-literal-replacement-alist' respectively.")
 Any substring matching the car of a cell in
 `parsebib-TeX-markup-replace-alist' is replaced with the
 corresponding cdr (if the cdr is a string), or with the result of
-calling the cdr on the match (if it is a function).  This is done
-with `replace-regexp-in-string', which see for details."
+calling the cdr on the match (if it is a function)."
   (let ((case-fold-search nil))
     (cl-loop for (replacement . pattern) in parsebib-TeX-markup-replacement-alist
              do (setq string (replace-regexp-in-string
@@ -453,7 +452,7 @@ if a matching delimiter was found."
     ;; If forward-sexp does not result in an error, we want to return t.
     t))
 
-(defun parsebib--parse-bib-value (limit &optional strings replace-TeX)
+(defun parsebib--parse-bib-value (limit &optional strings replace-TeX collapse-whitespace)
   "Parse value at point.
 A value is either a field value or a @String expansion.  Return
 the value as a string.  No parsing is done beyond LIMIT, but note
@@ -461,13 +460,15 @@ that parsing may stop well before LIMIT.
 
 STRINGS, if non-nil, is a hash table of @String definitions.
 @String abbrevs in the value to be parsed are then replaced with
-their expansions.  Additionally, newlines in field values are
-removed, white space is reduced to a single space and braces or
-double quotes around field values are removed.
+their expansions.  Additionally, newlines in field values and
+braces or double quotes around field values are removed.
 
 REPLACE-TEX indicates whether TeX markup should be replaced with
 ASCII/Unicode characters.  See the variable
-`parsebib-TeX-markup-replace-alist' for details."
+`parsebib-TeX-markup-replace-alist' for details.
+
+If COLLAPSE-WHITESPACE in non-nil, sequences of whitespace in the
+field value are collapsed into a single space."
   (let (res)
     (while (and (< (point) limit)
                 (not (looking-at-p ",")))
@@ -483,7 +484,7 @@ ASCII/Unicode characters.  See the variable
         (goto-char (match-end 0)))
        (t (forward-char 1)))) ; So as not to get stuck in an infinite loop.
     (setq res (if strings
-                  (string-join (parsebib--expand-strings (nreverse res) strings))
+                  (string-join (parsebib--expand-strings (nreverse res) strings collapse-whitespace))
                 (string-join (nreverse res) " # ")))
     (if replace-TeX
         (parsebib-clean-TeX-markup res)
@@ -493,15 +494,17 @@ ASCII/Unicode characters.  See the variable
 ;; Expanding stuff ;;
 ;;;;;;;;;;;;;;;;;;;;;
 
-(defun parsebib--expand-strings (strings abbrevs)
-  "Expand strings in STRINGS using expansions in ABBREVS.
+(defun parsebib--expand-strings (strings abbrevs collapse-whitespace)
+  "Expand @Strings abbreviations in STRINGS using expansions in ABBREVS.
 STRINGS is a list of strings.  If a string in STRINGS has an
 expansion in hash table ABBREVS, replace it with its expansion.
 Otherwise, if the string is enclosed in braces {} or double
-quotes \"\", remove the delimiters.  In addition, newlines and
-multiple spaces in the string are replaced with a single space."
+quotes \"\", remove the delimiters.  In addition, if
+COLLAPSE-WHITESPACE is non-nil, sequences of whitespace in the
+string are replaced with a single space."
   (mapcar (lambda (str)
-            (setq str (replace-regexp-in-string "[ \t\n\f[:space:]]+" " " str))
+            (when collapse-whitespace
+              (setq str (replace-regexp-in-string "[[:space:]\t\n\f]+" " " str)))
             (cond
              ((gethash str abbrevs))
              ((string-match "\\`[\"{]\\(.*?\\)[\"}]\\'" str)
@@ -631,7 +634,7 @@ point."
       (if (looking-at-p "[[:space:]]*[\(\{]")
           (progn (skip-chars-forward "[:space:]")
                  (parsebib--match-paren-forward))
-        (goto-char (point-at-eol)))
+        (goto-char (line-end-position)))
       (buffer-substring-no-properties beg (point)))))
 
 (defun parsebib-read-string (&optional pos strings)
@@ -769,10 +772,12 @@ ASCII/Unicode characters.  See the variable
       (if (parsebib--looking-at-goto-end (concat "\\(" parsebib--bibtex-identifier "\\)[[:space:]]*=[[:space:]]*") 1)
           (let* ((field (buffer-substring-no-properties beg (point)))
                  (replace-TeX (and replace-TeX
-                                   (not (member-ignore-case field parsebib-clean-TeX-markup-excluded-fields)))))
+                                   (not (member-ignore-case field parsebib-postprocessing-excluded-fields))))
+                 (collapse-whitespace (and strings
+                                           (not (member-ignore-case field parsebib-postprocessing-excluded-fields)))))
             (if (or (not fields)
                     (member-ignore-case field fields))
-                (cons field (parsebib--parse-bib-value limit strings replace-TeX))
+                (cons field (parsebib--parse-bib-value limit strings replace-TeX collapse-whitespace))
               (parsebib--parse-bib-value limit) ; Skip over the field value.
               :ignore)))))) ; Ignore this field but keep the `cl-loop' in `parsebib-read-entry' going.
 
@@ -834,7 +839,14 @@ have to be empty.  It may contain entries from a previous parse.
 
 If STRINGS is non-nil, it should be a hash table of string
 definitions, which are used to expand abbreviations used in the
-entries.
+entries.  In addition, if STRINGS is set, sequences of whitespace
+in field values are collapsed into a single space, field values
+are unquoted (i.e., the double quotes or braces around them are
+removed), and TeX markup is prettified (see
+`parsebib-clean-TeX-markup' for details).  Note that @String
+expansion, collapsing of whitespace and prettifying TeX markup
+are not applied to fields listed in
+`parsebib-postprocessing-excluded-fields', but unquoting is.
 
 If INHERITANCE is non-nil, cross-references in the entries are
 resolved: if the crossref field of an entry points to an entry
@@ -869,7 +881,7 @@ FIELDS is nil, all fields are returned."
              for entry-type = (parsebib-find-next-item)
              while entry-type do
              (unless (member-ignore-case entry-type '("preamble" "string" "comment"))
-               (setq entry (parsebib-read-entry entry-type nil strings fields))
+               (setq entry (parsebib-read-entry entry-type nil strings fields (not (null strings))))
                (if entry
                    (puthash (cdr (assoc-string "=key=" entry)) entry entries))))
     (when inheritance
@@ -906,7 +918,12 @@ function `equal', the @String definitions are stored in it.
 
 If EXPAND-STRINGS is non-nil, abbreviations in the entries and
 @String definitions are expanded using the @String definitions
-already in STRINGS.
+already in STRINGS.  In addition, sequences of whitespace in
+field values are collapsed into a single space and field values
+are unquoted, i.e., the double quotes or braces around them are
+removed.  Note that @String expansion, collapsing of whitespace
+and prettifying TeX markup are not applied to fields listed in
+`parsebib-postprocessing-excluded-fields', but unquoting is.
 
 If INHERITANCE is non-nil, cross-references in the entries are
 resolved: if the crossref field of an entry points to an entry
@@ -932,18 +949,18 @@ FIELDS is nil, all fields are returned.
 REPLACE-TEX indicates whether TeX markup should be replaced with
 ASCII/Unicode characters.  See the variable
 `parsebib-TeX-markup-replace-alist' for details."
-  (save-excursion
-    (goto-char (point-min))
-    (or (and (hash-table-p entries)
-             (eq (hash-table-test entries) 'equal))
-        (setq entries (make-hash-table :test #'equal)))
-    (or (and (hash-table-p strings)
-             (eq (hash-table-test strings) 'equal))
-        (setq strings (make-hash-table :test #'equal)))
-    (let ((dialect (or (parsebib-find-bibtex-dialect)
-                       bibtex-dialect
-                       'BibTeX))
-          preambles comments)
+  (or (and (hash-table-p entries)
+           (eq (hash-table-test entries) 'equal))
+      (setq entries (make-hash-table :test #'equal)))
+  (or (and (hash-table-p strings)
+           (eq (hash-table-test strings) 'equal))
+      (setq strings (make-hash-table :test #'equal)))
+  (let ((dialect (or (parsebib-find-bibtex-dialect)
+                     bibtex-dialect
+                     'BibTeX))
+        preambles comments)
+    (save-excursion
+      (goto-char (point-min))
       (cl-loop for item = (parsebib-find-next-item)
                while item do
                (cond
